package com.sun.org.apache.xml.internal.resolver.tools;

import com.sun.org.apache.xml.internal.resolver.helpers.FileURL;
import java.io.InputStream;
import org.xml.sax.AttributeList;
import org.xml.sax.Locator;
import java.net.MalformedURLException;
import java.util.Locale;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import java.io.IOException;
import org.xml.sax.InputSource;
import com.sun.org.apache.xml.internal.resolver.Catalog;
import javax.xml.parsers.SAXParserFactory;
import jdk.xml.internal.JdkXmlUtils;
import java.net.URL;
import com.sun.org.apache.xml.internal.resolver.CatalogManager;
import javax.xml.parsers.SAXParser;
import org.xml.sax.EntityResolver;
import org.xml.sax.DocumentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.Parser;

public class ResolvingParser implements Parser, DTDHandler, DocumentHandler, EntityResolver
{
    public static boolean namespaceAware;
    public static boolean validating;
    private static final boolean suppressExplanation = false;
    private SAXParser saxParser;
    private Parser parser;
    private DocumentHandler documentHandler;
    private DTDHandler dtdHandler;
    private CatalogManager catalogManager;
    private CatalogResolver catalogResolver;
    private CatalogResolver piCatalogResolver;
    private boolean allowXMLCatalogPI;
    private URL baseURL;
    
    public ResolvingParser() {
        this.saxParser = null;
        this.parser = null;
        this.documentHandler = null;
        this.dtdHandler = null;
        this.catalogManager = CatalogManager.getStaticManager();
        this.catalogResolver = null;
        this.piCatalogResolver = null;
        this.allowXMLCatalogPI = false;
        this.baseURL = null;
        this.initParser();
    }
    
    public ResolvingParser(CatalogManager manager) {
        this.saxParser = null;
        this.parser = null;
        this.documentHandler = null;
        this.dtdHandler = null;
        this.catalogManager = CatalogManager.getStaticManager();
        this.catalogResolver = null;
        this.piCatalogResolver = null;
        this.allowXMLCatalogPI = false;
        this.baseURL = null;
        this.catalogManager = manager;
        this.initParser();
    }
    
    private void initParser() {
        this.catalogResolver = new CatalogResolver(this.catalogManager);
        SAXParserFactory spf = JdkXmlUtils.getSAXFactory(this.catalogManager.overrideDefaultParser());
        spf.setNamespaceAware(ResolvingParser.namespaceAware);
        spf.setValidating(ResolvingParser.validating);
        try {
            this.saxParser = spf.newSAXParser();
            this.parser = this.saxParser.getParser();
            this.documentHandler = null;
            this.dtdHandler = null;
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public Catalog getCatalog() {
        return this.catalogResolver.getCatalog();
    }
    
    @Override
    public void parse(InputSource input) throws IOException, SAXException {
        this.setupParse(input.getSystemId());
        try {
            this.parser.parse(input);
        }
        catch (InternalError ie) {
            this.explain(input.getSystemId());
            throw ie;
        }
    }
    
    @Override
    public void parse(String systemId) throws IOException, SAXException {
        this.setupParse(systemId);
        try {
            this.parser.parse(systemId);
        }
        catch (InternalError ie) {
            this.explain(systemId);
            throw ie;
        }
    }
    
    @Override
    public void setDocumentHandler(DocumentHandler handler) {
        this.documentHandler = handler;
    }
    
    @Override
    public void setDTDHandler(DTDHandler handler) {
        this.dtdHandler = handler;
    }
    
    @Override
    public void setEntityResolver(EntityResolver resolver) {
    }
    
    @Override
    public void setErrorHandler(ErrorHandler handler) {
        this.parser.setErrorHandler(handler);
    }
    
    @Override
    public void setLocale(Locale locale) throws SAXException {
        this.parser.setLocale(locale);
    }
    
    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        if (this.documentHandler != null) {
            this.documentHandler.characters(ch, start, length);
        }
    }
    
    @Override
    public void endDocument() throws SAXException {
        if (this.documentHandler != null) {
            this.documentHandler.endDocument();
        }
    }
    
    @Override
    public void endElement(String name) throws SAXException {
        if (this.documentHandler != null) {
            this.documentHandler.endElement(name);
        }
    }
    
    @Override
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        if (this.documentHandler != null) {
            this.documentHandler.ignorableWhitespace(ch, start, length);
        }
    }
    
    @Override
    public void processingInstruction(String target, String pidata) throws SAXException {
        if (target.equals("oasis-xml-catalog")) {
            URL catalog = null;
            String data = pidata;
            int pos = data.indexOf("catalog=");
            if (pos >= 0) {
                data = data.substring(pos + 8);
                if (data.length() > 1) {
                    String quote = data.substring(0, 1);
                    data = data.substring(1);
                    pos = data.indexOf(quote);
                    if (pos >= 0) {
                        data = data.substring(0, pos);
                        try {
                            if (this.baseURL != null) {
                                catalog = new URL(this.baseURL, data);
                            }
                            else {
                                catalog = new URL(data);
                            }
                        }
                        catch (MalformedURLException ex) {}
                    }
                }
            }
            if (this.allowXMLCatalogPI) {
                if (this.catalogManager.getAllowOasisXMLCatalogPI()) {
                    this.catalogManager.debug.message(4, "oasis-xml-catalog PI", pidata);
                    if (catalog != null) {
                        try {
                            this.catalogManager.debug.message(4, "oasis-xml-catalog", catalog.toString());
                            if (this.piCatalogResolver == null) {
                                this.piCatalogResolver = new CatalogResolver(true);
                            }
                            this.piCatalogResolver.getCatalog().parseCatalog(catalog.toString());
                        }
                        catch (Exception e) {
                            this.catalogManager.debug.message(3, "Exception parsing oasis-xml-catalog: " + catalog.toString());
                        }
                    }
                    else {
                        this.catalogManager.debug.message(3, "PI oasis-xml-catalog unparseable: " + pidata);
                    }
                }
                else {
                    this.catalogManager.debug.message(4, "PI oasis-xml-catalog ignored: " + pidata);
                }
            }
            else {
                this.catalogManager.debug.message(3, "PI oasis-xml-catalog occurred in an invalid place: " + pidata);
            }
        }
        else if (this.documentHandler != null) {
            this.documentHandler.processingInstruction(target, pidata);
        }
    }
    
    @Override
    public void setDocumentLocator(Locator locator) {
        if (this.documentHandler != null) {
            this.documentHandler.setDocumentLocator(locator);
        }
    }
    
    @Override
    public void startDocument() throws SAXException {
        if (this.documentHandler != null) {
            this.documentHandler.startDocument();
        }
    }
    
    @Override
    public void startElement(String name, AttributeList atts) throws SAXException {
        this.allowXMLCatalogPI = false;
        if (this.documentHandler != null) {
            this.documentHandler.startElement(name, atts);
        }
    }
    
    @Override
    public void notationDecl(String name, String publicId, String systemId) throws SAXException {
        this.allowXMLCatalogPI = false;
        if (this.dtdHandler != null) {
            this.dtdHandler.notationDecl(name, publicId, systemId);
        }
    }
    
    @Override
    public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException {
        this.allowXMLCatalogPI = false;
        if (this.dtdHandler != null) {
            this.dtdHandler.unparsedEntityDecl(name, publicId, systemId, notationName);
        }
    }
    
    @Override
    public InputSource resolveEntity(String publicId, String systemId) {
        this.allowXMLCatalogPI = false;
        String resolved = this.catalogResolver.getResolvedEntity(publicId, systemId);
        if (resolved == null && this.piCatalogResolver != null) {
            resolved = this.piCatalogResolver.getResolvedEntity(publicId, systemId);
        }
        if (resolved != null) {
            try {
                InputSource iSource = new InputSource(resolved);
                iSource.setPublicId(publicId);
                URL url = new URL(resolved);
                InputStream iStream = url.openStream();
                iSource.setByteStream(iStream);
                return iSource;
            }
            catch (Exception e) {
                this.catalogManager.debug.message(1, "Failed to create InputSource (" + e.toString() + ")", resolved);
                return null;
            }
        }
        else {
            return null;
        }
    }
    
    private void setupParse(String systemId) {
        this.allowXMLCatalogPI = true;
        this.parser.setEntityResolver(this);
        this.parser.setDocumentHandler(this);
        this.parser.setDTDHandler(this);
        URL cwd = null;
        try {
            cwd = FileURL.makeURL("basename");
        }
        catch (MalformedURLException mue) {
            cwd = null;
        }
        try {
            this.baseURL = new URL(systemId);
        }
        catch (MalformedURLException mue) {
            if (cwd != null) {
                try {
                    this.baseURL = new URL(cwd, systemId);
                }
                catch (MalformedURLException mue2) {
                    this.baseURL = null;
                }
            }
            else {
                this.baseURL = null;
            }
        }
    }
    
    private void explain(String systemId) {
        System.out.println("Parser probably encountered bad URI in " + systemId);
        System.out.println("For example, replace '/some/uri' with 'file:/some/uri'.");
    }
    
    static {
        ResolvingParser.namespaceAware = true;
        ResolvingParser.validating = false;
    }
}
